@@ -20,6 +20,8 @@
 ## Two roots are the same if the difference is less than the PRECISION
 PRECISION <- 1e-3
 
+## Maximum frequencies of a histogram
+MAXLENGTH <- 10000000
 
 ### interpolating for species accumulation curve with minimum count
 ### ss step size
@@ -134,239 +136,6 @@ generating.ps <- function(n, j, mt) {
 }
 
 
-### species accum curves based on parital fraction expansion of
-### rational function approximation to E(S_1(t)) / t
-### CHAO: the main function
-preseqR.pf.mincount <- function(n, mt = 100, ss = NULL, 
-                                max.extrapolation = NULL, r=1)
-{
-  # check the input format of the histogram
-  checking.hist(n)
-  ## setting the diagonal value
-  di <- 0
-  ## minimum required number of terms of power series in order to construct
-  ## a continued fraction approximation
-  MIN_REQUIRED_TERMS <- 4
-
-  n[, 2] <- as.numeric(n[, 2])
-  ## total individuals
-  total.sample <- n[, 1] %*% n[, 2]
-
-  ## set step.size as the size of the initial experiment if it is undefined
-  if (is.null(ss)) {
-    ss <- total.sample
-    step.size <- ss
-  } else if (ss < 1) {
-    write("The step size is too small", stderr())
-    return()
-  } else {
-    step.size <- ss
-  }
-
-  ## no interpolation if step.size is larger than the size of the iniital experiment
-  ## set the starting sample size as the step.size
-  if (step.size > total.sample) {
-    yield.estimates <- NULL
-
-    ## starting sample size for extrapolation
-    starting.size <- step.size
-  } else {
-      ## interpolating when sample size is no more than total sample size
-      ## and setting the size of the sample for an initial extrapolation
-      yield.estimates <- lapply(r, function(x) {preseqR.interpolate.mincount(step.size, n, x)})
-
-      ## starting sample size for extrapolation
-      starting.size <- ( floor(total.sample/step.size) + 1 )*step.size
-  }
-
-  if (is.null(max.extrapolation)) {
-    ## extrapolation 100 times if it is undefined; 100 is a magic number
-    max.extrapolation <- 100*total.sample
-  }
-
-  ## constructing the power series
-  PS.coeffs <- generating.ps(n, 1, mt=mt)
-
-  if (is.null(PS.coeffs)) {
-    write("the size of the initial experiment is insufficient", stderr())
-    return(NULL)
-  }
-
-  ## constrain the continued fraction approximation with even terms
-  ## asymptotically ~ C / t
-  mt <- min(mt, length(PS.coeffs))
-  mt <- mt - (mt %% 2)
-  PS.coeffs <- PS.coeffs[ 1:mt ]
-
-  ## check whether sample size is sufficient
-  if (mt < MIN_REQUIRED_TERMS)
-  {
-    m <- paste("max count before zero is les than min required count (4)",
-               " sample not sufficiently deep or duplicates removed", sep = ',')
-    write(m, stderr())
-    return(NULL)
-  }
-
-  ## construct a continued fraction approximation including as many as possible
-  ## terms
-  valid <- FALSE
-  DE <- seq(mt, MIN_REQUIRED_TERMS, by=-2)
-  for (de in DE) {
-    ## continued fraction approximation to a power series
-    out <- .C('c_PS2CF', as.integer(di), 
-              as.integer(de), as.double(PS.coeffs[1:de]), 
-              as.integer(length(PS.coeffs[1:de])),
-              ps.coeffs = as.double(vector(mode = 'numeric', length=MAXLENGTH)),
-              ps.coeffs.l = as.integer(0),
-              cf.coeffs = as.double(vector(mode = 'numeric', length=MAXLENGTH)),
-              cf.coeffs.l = as.integer(0),
-              offset.coeffs =as.double(vector(mode='numeric',length=MAXLENGTH)),
-              diagonal.idx = as.integer(0),
-              degree = as.integer(0), is.valid = as.integer(0));
-    if (out$is.valid) {break}
-  }
-  if (out$is.valid) {
-    length(out$ps.coeffs) <- out$ps.coeffs.l
-    length(out$cf.coeffs) <- out$cf.coeffs.l
-    length(out$offset.coeffs) <- as.integer(abs(out$diagonal.idx))
-    CF.space <- list(out$ps.coeffs, out$cf.coeffs, out$offset.coeffs, 
-                     out$diagonal.idx, out$degree)
-    names(CF.space) <- c('ps.coeffs', 'cf.coeffs', 'offset.coeffs', 'diagonal.idx',
-                         'degree')
-    DE = seq(CF.space$degree, MIN_REQUIRED_TERMS, by=-2)
-
-    for (de in DE) {
-      CF <- list(CF.space$ps.coeffs[1:de], CF.space$cf.coeffs[1:de],
-                 CF.space$offset.coeffs, CF.space$diagonal.idx, de)
-      names(CF) <- c('ps.coeffs', 'cf.coeffs', 'offset.coeffs', 'diagonal.idx',
-                     'degree')
-      class(CF) <- 'CF'
-      ## convert the continued fraction to the RFA 
-      RF <- CF2RFA(CF)
-      RF[[1]] <- RF[[1]] / polynomial(c(0, 1))
-
-      ## solving roots
-      numer.roots <- solve(RF[[1]])
-      denom.roots <- solve(RF[[2]])
-      ## seperating roots by their real parts
-      numer.roots.neg <- numer.roots[which(Re(numer.roots) < 0)]
-      numer.roots.pos <- numer.roots[which(Re(numer.roots) >= 0)]
-      denom.roots.neg <- denom.roots[which(Re(denom.roots) < 0)]
-      denom.roots.pos <- denom.roots[which(Re(denom.roots) >= 0)]
-
-      ## record roots in the numerator that are significantly similar to
-      ## roots in the denominator
-      tmp.roots <- c()
-
-      ## simplify the rational function approximation
-      ## two roots are same if the difference is less than the 
-      ## predefined PRECISION
-      if (length(numer.roots.pos) > 0) {
-        for (i in 1:length(numer.roots.pos)) {
-          if (length(denom.roots.pos) > 0) {
-            d <- Mod(denom.roots.pos - numer.roots.pos[i])
-            for (j in 1:length(d)) {
-              if (d[j] < PRECISION) {
-                denom.roots.pos <- denom.roots.pos[-j]
-                tmp.roots <- c(tmp.roots, numer.roots.pos[i])
-                break
-              }
-            }
-          }
-        }
-      }
-
-      ## roots in simplified RFA
-      numer.roots <- numer.roots[!numer.roots %in% tmp.roots]
-      denom.roots <- c(denom.roots.neg, denom.roots.pos)
-
-      ## convert roots from t - 1 to t
-      roots <- denom.roots + 1
-      ## max.range = max.extrapolation / total.sample
-      ## pacman rule checking
-      if (length(which(roots == 0)) || length(which(Re(roots) > 0))) {
-        next
-      } else {
-        poly.numer <- as.function(poly.from.roots(numer.roots))
-        l <- length(denom.roots)
-        ## treat polynomials in the rational function to be monic
-        ## the difference to the original RFA is a multiplier C
-
-        ## c_i in the estimator
-        coef <- sapply(1:l, function(x) {
-          poly.numer(denom.roots[x]) / prod(denom.roots[x] - denom.roots[-x])})
-        ## check whether the estimator is non-decreased
-        ## NOTE: it only checks for t >= 1 !!!
-        deriv.f <- function(t) {
-          Re(sapply(t, function(x) {-(coef*roots) %*% ( 1 / ((x-denom.roots)^2))}))} 
-        if (length(which( deriv.f(seq(0.05, as.double(max.extrapolation / total.sample), by=0.05)) < 0 ) != 0)) {
-          next
-        }
-        ## the estimator passes the requirement
-        valid <- TRUE
-        ## calculate the constant C
-        C <- coef(RF[[1]])[length(coef(RF[[1]]))] / 
-             coef(RF[[2]])[length(coef(RF[[2]]))]
-        ## species accum curves with minimum count r
-        ## using parital fraction expansion
-        denom.roots <- denom.roots + 1
-        coef <- coef * C
-        mincount.f.elements <- list(coef, denom.roots)
-        mincount.accum.curve.f <- lapply(r, function(x) {
-            function(t) { Re(sapply(t, function(y) {coef %*% ( y / (y-denom.roots) )^x}))}})
-        break
-      }
-    }
-  }
-
-
-  ## if the sample size is larger than max.extrapolation
-  ## stop extrapolation
-  ## MINOR.correction prevents machinary precision from biasing comparison
-  ## result
-  if (starting.size > (max.extrapolation + MINOR.correction))
-  {
-    if (length(yield.estimates) == 0) {
-      return(NULL)
-    } else {
-      l <- 1:length(r)
-      result <- lapply(l, function(x) {
-          estimates <- yield.estimates[[x]]
-          colnames(estimates) <- c("sample.size", paste("yield.estimates(r=", r[x], ")", sep=""))
-          })
-      return(result)
-    }
-  }
-
-  ## valid is true if existing a RFA that satisfies pacman rule
-  if (valid == FALSE) {
-    return(NULL)
-  }
-
-  ## extrapolation for experiment with large sample size
-  start <- starting.size / total.sample
-  end <- (max.extrapolation + MINOR.correction) / total.sample
-  step <- step.size / total.sample
-
-  ## extrapolating for the general accumulation curves
-  l <- 1:length(r)
-  extrap <- lapply(l, function(x) {
-      mincount.accum.curve.f[[x]](seq(start, end, by=step))})
-
-  result <- lapply(l, function(x) {
-      ## combine results from interpolation and extrapolation
-      estimates <- c(yield.estimates[[x]][, 2], extrap[[x]])
-      index <- as.double(step.size) * (1: length(estimates))
-      ## put index and estimated yields together into a two-colunm matrix
-      estimates <- matrix(c(index, estimates), ncol = 2, byrow = FALSE)
-      colnames(estimates) <- c("sample.size", paste("yield.estimates(r=", r[x], ")", sep=""))
-      estimates
-      })
-
-  list(PF.elements=mincount.f.elements, yield.estimates=result)
-}
-
-
 ## species accum curves based on parital fraction expansion
 ## the function is used whenever count frequency 1 is unavaible or the sample
 ## size is saturated
@@ -377,6 +146,8 @@ general.preseqR.pf.mincount <- function(n, mt = 100, ss = NULL,
                                         max.extrapolation = NULL, 
                                         r=1, start.freq=1)
 {
+  MINOR.correction <- 1e-1
+  BOOTSTRAP.factor <- 0.4
   # check the input format of the histogram
   checking.hist(n)
   ## setting the diagonal value
@@ -604,15 +375,8 @@ general.preseqR.pf.mincount <- function(n, mt = 100, ss = NULL,
 }
 
 
-### species accum curves based on parital fraction expansion of
-### rational function approximation to E(S_1(t)) / t
-### CHAO: the main function
-preseqR.pf.mincount.bootstrap <- function(n, bootstrap.times = 100, mt = 100,
-                                          ss = NULL, max.extrapolation = NULL, 
-                                          conf = 0.95, r=1)
+ds.mincount <- function(n, r=1, mt=100)
 {
-  # check the input format of the histogram
-  checking.hist(n)
   ## setting the diagonal value
   di <- 0
   ## minimum required number of terms of power series in order to construct
@@ -620,116 +384,196 @@ preseqR.pf.mincount.bootstrap <- function(n, bootstrap.times = 100, mt = 100,
   MIN_REQUIRED_TERMS <- 4
 
   n[, 2] <- as.numeric(n[, 2])
-  ## total individuals
-  total.sample <- as.double(n[, 1] %*% n[, 2])
 
-  ## set step.size as the size of the initial experiment if it is undefined
-  if (is.null(ss)) {
-    ss <- total.sample
-    step.size <- ss
-  } else if (ss < 1) {
-    write("The step size is too small", stderr())
-    return()
-  } else {
-    step.size <- ss
+  ## constructing the power series
+  PS.coeffs <- generating.ps(n, 1, mt=mt)
+
+  if (is.null(PS.coeffs)) {
+    write("the size of the initial experiment is insufficient", stderr())
+    return(NULL)
   }
 
-  if (is.null(max.extrapolation)) {
-    ## extrapolation 100 times if it is undefined; 100 is a magic number
-    max.extrapolation <- 100*total.sample
-  }
-  
-  ## record second columns of resampled histograms                                  
-  re.hist.second.col <- matrix(data = 0, nrow = length(n[, 1]),
-                               ncol = MULTINOMIAL.SAMPLE.TIMES) 
+  ## constrain the continued fraction approximation with even terms
+  ## asymptotically ~ C / t
+  mt <- min(mt, length(PS.coeffs))
+  mt <- mt - (mt %% 2)
+  PS.coeffs <- PS.coeffs[ 1:mt ]
 
-  ## the number of resampling times                                                 
-  counter <- 0
-
-  ## upperbound of times of iterations for bootstrapping
-  upper.limit <- bootstrap.times/BOOTSTRAP.factor
-
-  yield.estimates <- array(list(NULL), length(r))
-
-  f <- function(x)
+  ## check whether sample size is sufficient
+  if (mt < 2)
   {
-    ## combine nonzero.index column and the second column to build a histogram
-    ## table
-    hist.table <- matrix(c(n[, 1], x), ncol = 2, byrow = FALSE)
-    preseqR.pf.mincount(hist.table, mt=mt, ss=step.size, 
-        max.extrapolation=max.extrapolation, r=r)
+    m <- paste("max count before zero is les than min required count (2)",
+               " sample not sufficiently deep or duplicates removed", sep = ',')
+    write(m, stderr())
+    return(NULL)
   }
-  
-  BOOTSTRAP.times <- bootstrap.times
 
-  while (bootstrap.times > 0) {
-    ## do sampling with replacement
-    ## re.hist.second.col saves the second columns of each resampled histogram
-    re.hist.second.col <- rmultinom(MULTINOMIAL.SAMPLE.TIMES, sum(n[, 2]), n[, 2])
+  ## construct a continued fraction approximation including as many as possible
+  ## terms
+  valid <- FALSE
+  if (mt >=  MIN_REQUIRED_TERMS ) {
+    DE <- seq(mt, MIN_REQUIRED_TERMS, by=-2)
+    for (de in DE) {
+      ## continued fraction approximation to a power series
+      out <- .C('c_PS2CF', as.integer(di), 
+              as.integer(de), as.double(PS.coeffs[1:de]), 
+              as.integer(length(PS.coeffs[1:de])),
+              ps.coeffs = as.double(vector(mode = 'numeric', length=MAXLENGTH)),
+              ps.coeffs.l = as.integer(0),
+              cf.coeffs = as.double(vector(mode = 'numeric', length=MAXLENGTH)),
+              cf.coeffs.l = as.integer(0),
+              offset.coeffs =as.double(vector(mode='numeric',length=MAXLENGTH)),
+              diagonal.idx = as.integer(0),
+              degree = as.integer(0), is.valid = as.integer(0));
+      if (out$is.valid) {break}
+    }
+    if (out$is.valid) {
+      length(out$ps.coeffs) <- out$ps.coeffs.l
+      length(out$cf.coeffs) <- out$cf.coeffs.l
+      length(out$offset.coeffs) <- as.integer(abs(out$diagonal.idx))
+      CF.space <- list(out$ps.coeffs, out$cf.coeffs, out$offset.coeffs, 
+                     out$diagonal.idx, out$degree)
+      names(CF.space) <- c('ps.coeffs', 'cf.coeffs', 'offset.coeffs', 'diagonal.idx',
+                         'degree')
+      DE = seq(CF.space$degree, MIN_REQUIRED_TERMS, by=-2)
 
-    ## estimate for each histogram
-    out <- apply(re.hist.second.col, 2, f)
+      for (de in DE) {
+        CF <- list(CF.space$ps.coeffs[1:de], CF.space$cf.coeffs[1:de],
+                   CF.space$offset.coeffs, CF.space$diagonal.idx, de)
+        names(CF) <- c('ps.coeffs', 'cf.coeffs', 'offset.coeffs', 'diagonal.idx',
+                     'degree')
+        class(CF) <- 'CF'
+        ## convert the continued fraction to the RFA 
+        RF <- CF2RFA(CF)
+        RF[[1]] <- RF[[1]] / polynomial(c(0, 1))
 
-    ## eliminate NULL items in results
-    out[sapply(out, is.null)] <- NULL
-    ## extract yields estimation from each estimation result.
-    yields <- lapply(out, function(x) x$yield.estimates)
+        ## solving roots
+        numer.roots <- solve(RF[[1]])
+        denom.roots <- solve(RF[[2]])
+        ## seperating roots by their real parts
+        numer.roots.neg <- numer.roots[which(Re(numer.roots) < 0)]
+        numer.roots.pos <- numer.roots[which(Re(numer.roots) >= 0)]
+        denom.roots.neg <- denom.roots[which(Re(denom.roots) < 0)]
+        denom.roots.pos <- denom.roots[which(Re(denom.roots) >= 0)]
 
-    if ( length(yields) > 0 )
-    {
-      ## update sampling status
-      success.times <- length(yields)
-      bootstrap.times <- bootstrap.times - success.times
-      for (i in 1:length(r)) {
-        for (j in 1:length(yields)) {
-          yield.estimates[[i]] <- cbind(yield.estimates[[i]], yields[[j]][[i]][, 2])
+        ## record roots in the numerator that are significantly similar to
+        ## roots in the denominator
+        tmp.roots <- c()
+
+        ## simplify the rational function approximation
+        ## two roots are same if the difference is less than the 
+        ## predefined PRECISION
+        if (length(numer.roots.pos) > 0) {
+          for (i in 1:length(numer.roots.pos)) {
+            if (length(denom.roots.pos) > 0) {
+              d <- Mod(denom.roots.pos - numer.roots.pos[i])
+              for (j in 1:length(d)) {
+                if (d[j] < PRECISION) {
+                  denom.roots.pos <- denom.roots.pos[-j]
+                  tmp.roots <- c(tmp.roots, numer.roots.pos[i])
+                  break
+                }
+              }
+            }
+          }
+        }
+
+        ## roots in simplified RFA
+        numer.roots <- numer.roots[!numer.roots %in% tmp.roots]
+        denom.roots <- c(denom.roots.neg, denom.roots.pos)
+
+        ## convert roots from t - 1 to t
+        roots <- denom.roots + 1
+        ## pacman rule checking
+        if (length(which(roots == 0)) || length(which(Re(roots) > 0))) {
+          next
+        } else {
+          poly.numer <- as.function(poly.from.roots(numer.roots))
+          l <- length(denom.roots)
+          ## treat polynomials in the rational function to be monic
+          ## the difference to the original RFA is a multiplier C
+
+          ## c_i in the estimator
+          coef <- sapply(1:l, function(x) {
+            poly.numer(denom.roots[x]) / prod(denom.roots[x] - denom.roots[-x])})
+          ## check whether the estimator is non-decreased
+          ## NOTE: it only checks for t >= 1 !!!
+          deriv.f <- function(t) {
+            Re(sapply(t, function(x) {-(coef*roots) %*% ( 1 / ((x-denom.roots)^2))}))} 
+          if (length(which( deriv.f(seq(0.05, 100, by=0.05)) < 0 ) != 0)) {
+            next
+          }
+          ## the estimator passes the requirement
+          valid <- TRUE
+          ## calculate the constant C
+          C <- coef(RF[[1]])[length(coef(RF[[1]]))] / 
+               coef(RF[[2]])[length(coef(RF[[2]]))]
+          ## species accum curves with minimum count r
+          ## using parital fraction expansion
+          denom.roots <- denom.roots + 1
+          coef <- coef * C
+          f.mincount <- function(t) {
+            sapply(r, function(x) {
+              Re(coef %*% (t / (t - denom.roots))^x)})}
+          break
         }
       }
     }
-
-    ## update sampling tmes
-    counter <- counter + MULTINOMIAL.SAMPLE.TIMES
-    if (counter > upper.limit)
-      break;
   }
-
-  ## enough successful sampling
-  if (bootstrap.times <= 0) {
-    
-    if (BOOTSTRAP.times < 30) {
-      warning("The confidence interval is not reliable because of insufficient iterations of bootstrapping.
-  Set the variable bootstrap.times at least 30 in order to construct confidence intervals")}
-
-    ## the number of sampled points for complexity curve
-    n <- dim(yield.estimates[[1]])[1]
-
-    ## sample sizes
-    index <- as.double(step.size) * ( 1:n )
-    result <- array(list(NULL), length(r))
-
-    for (i in 1:length(r)) {
-
-      # median values are used as complexity curve
-      median.estimate <- apply(yield.estimates[[i]], 1, median)
-      variance <- apply(yield.estimates[[i]], 1, var)
-
-      # confidence interval based on lognormal
-      if (conf <= 0 && conf >= 1)
-        conf <- 0.95
-      C <- exp(qnorm((1 + conf) / 2.0) * sqrt(log(1.0 + variance / (median.estimate^2))))
-      left.interval <- median.estimate/C
-      right.interval <- median.estimate*C
-
-      ## combine results and output a matrix
-      result[[i]] <- matrix(c(index, median.estimate, left.interval, right.interval), 
-                            ncol = 4, byrow = FALSE)
-      lower.ci <- sprintf('lower.%.2fCI', conf)
-      upper.ci <- sprintf('uppper.%.2fCI', conf)
-      colnames(result[[i]]) <- c('sample.size', paste("yield.estimates(r=", r[i], ")", sep=""), lower.ci, upper.ci)
-    }
-    return(result)
-  } else {
-      write("fail to bootstrap!", stderr())
-      return(NULL)
+  if (valid==FALSE) {
+    s1 <- sum(n[, 2])
+    s2 <- sum(n[n[, 1] > 1, 2])
+    coef <- s1^2 / s2
+    denom.roots <- (s1 - s2) / s2
+    f.mincount <- function(t) {
+      sapply(r, function(x) {
+        coef * (t / (t + denom.roots))^x})}
+    f.mincount(1)
+    return(list(FUN=f.mincount, m=1, m.adjust=length(denom.roots), FUN.elements=list(coef=coef, roots=denom.roots)))
   }
+  f.mincount(1)
+  list(FUN=f.mincount, m=de / 2, m.adjust=length(denom.roots), FUN.elements=list(coef=coef, roots=denom.roots))
+}
+
+
+## nonparametric approach Deng & Smith 2016
+ds.mincount.bootstrap <- function(n, r=1, mt=100, times=100)
+{
+  n[, 2] <- as.numeric(n[, 2])
+  ## total individuals
+  total <- n[, 1] %*% n[, 2]
+
+  ## the number of resampling times
+  counter <- 0
+  ## returned function
+  f.mincount <- vector(length=times, mode="list")
+
+  ds.estimator <- function(n, r, mt, t.scale) {
+    f <- ds.mincount(n, r=r, mt=mt)
+    function(t) {f$FUN(t * t.scale)}
+  }
+
+  while (times > 0) {
+    n.bootstrap <- matrix(c(n[, 1], rmultinom(1, sum(n[, 2]), n[, 2])), ncol=2)
+    total.bootstrap <- n.bootstrap[, 1] %*% n.bootstrap[, 2]
+    t.scale <- total / total.bootstrap
+    f <-  ds.estimator(n.bootstrap, r=r, mt=mt, t.scale=t.scale) 
+    counter <- counter + 1
+
+    f.mincount[[times]] <- f
+    ## prevent later binding!!!
+    f.mincount[[times]](1)
+    times <- times - 1
+  }
+  f.estimator <- ds.mincount(n=n, r=r, mt=mt)
+  if (length(r) == 1) {
+    median.estimators <- function(t) {median( sapply(f.mincount, function(x) x(t)) )}
+    var.estimator <- function(t) {var( sapply(f.mincount, function(x) x(t)) )}
+  } else {
+    median.estimators <- function(t) {apply(sapply(f.mincount, function(x) x(t)), FUN=median, MARGIN=1)}
+    var.estimator <- function(t) {apply(sapply(f.mincount, function(x) x(t)), FUN=var, MARGIN=1)}
+  }
+  ## prevent later binding!!!
+  f.estimator$FUN(1); median.estimators(1); var.estimator(1)
+  return(list(FUN.nobootstrap=f.estimator, FUN.bootstrap=median.estimators, var=var.estimator))
 }
